#!/usr/bin/env python

"""
createRiverDatasets.py

main program to create river png images subsets for train/valid/test.
It uses numpy arrays with images, ALREADY CREATED by the script
loadRiverImages.py. The given image arrays are split using manual
editing of sample amount.

This is the first script to launch for dataset creation.

SEQUENCE OF SCRIPT EXECUTION:

1. (mandatory)	loadRiverImages.py
2. (mandatory)	createRiverDatasets.py
3. (mandatory)	riverAugmenting.py
4. (optional)	saveRiverImages.py

# Copyright (C) 2020-2022 DAME Project
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation;
# either version 3.0 of the License, or (at your option)
# any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to
# the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301 USA
#

WHO                 WHEN         WHAT
------------------- ----------	 -------------------------------------------------
M. Brescia			19/02/2020	 Created
-----------------------------------------------------------------------------------
"""

import os, numpy as np
import random

#load tensors from npy files:
in_dir = 'exp'

X = np.load(in_dir+os.sep+'X_riverbased.npy')
y = np.load(in_dir+os.sep+'y_riverbased.npy')
NAMES = np.load(in_dir+os.sep+'name_riverbased.npy')
mix_index = np.where(y==1)[0]
dendrite_index = np.where(y==0)[0]

imgTotal    = mix_index.shape[0] + dendrite_index.shape[0]
imgDendrite = dendrite_index.shape[0]
imgMixed    = mix_index.shape[0]
perc_train  = 0.7 # decimal percentage
perc_valid  = 0.1 # decimal percentage
random.seed(42)

n_dendrite_train = int(imgDendrite * perc_train)
n_mix_train = int(imgMixed * perc_train)
n_dendrite_valid = int(imgDendrite * perc_valid)
n_mix_valid = int(imgMixed * perc_valid)

# test data will result as difference from training+valid choice done
n_dendrite_test = dendrite_index.shape[0]-n_dendrite_train-n_dendrite_valid
n_mix_test = mix_index.shape[0]-n_mix_train-n_mix_valid

print("\nTotal imgs         : " + str(imgTotal))
print("Dendrite total imgs: " + str(dendrite_index.shape[0]))
print("Dendrite train imgs: " + str(n_dendrite_train))
print("Dendrite valid imgs: " + str(n_dendrite_valid))
print("Dendrite test imgs : " + str(n_dendrite_test))
print("Mixed total imgs   : " + str(mix_index.shape[0]))
print("Mixed train imgs   : " + str(n_mix_train))
print("Mixed valid imgs   : " + str(n_mix_valid))
print("Mixed test imgs    : " + str(n_mix_test) + "\n")

X_mix = X[mix_index]
NAMES_mix = NAMES[mix_index]
X_dendrite = X[dendrite_index]
NAMES_dendrite = NAMES[dendrite_index]

def split(Z, N, mix=True):
    if mix:
        n_train, n_valid, n_test = n_mix_train, n_mix_valid, n_mix_test
    else:
        n_train, n_valid, n_test = n_dendrite_train, n_dendrite_valid, n_dendrite_test

    index = random.sample(range(Z.shape[0]), n_train)
    mask = np.ones(Z.shape[0], dtype=bool)
    mask[index]=False
    Z_train = Z[index]
    N_train = N[index]
    Z = Z[mask]
    N = N[mask]

    index = random.sample(range(Z.shape[0]), n_valid)
    mask = np.ones(Z.shape[0], dtype=bool)
    mask[index]=False
    Z_valid = Z[index]
    N_valid = N[index]
    Z_test = Z[mask]
    N_test = N[mask]
    
    return Z_train, Z_valid, Z_test, N_train, N_valid, N_test

X_mix_train, X_mix_valid, X_mix_test, NAMES_mix_train, NAMES_mix_valid, NAMES_mix_test = split(X_mix, NAMES_mix, mix=True)
X_dendrite_train, X_dendrite_valid, X_dendrite_test, NAMES_dendrite_train, NAMES_dendrite_valid, NAMES_dendrite_test = split(X_dendrite, NAMES_dendrite, mix=False)  

#eventually you can save the results
tensor2save = [X_mix_train, X_mix_valid, X_mix_test, NAMES_mix_train, NAMES_mix_valid, NAMES_mix_test, X_dendrite_train, X_dendrite_valid, X_dendrite_test, NAMES_dendrite_train, NAMES_dendrite_valid, NAMES_dendrite_test]
names = ['X_mix_train.npy', 'X_mix_valid.npy', 'X_mix_test.npy', 'NAMES_mix_train.npy', 'NAMES_mix_valid.npy', 'NAMES_mix_test.npy', 'X_dendrite_train.npy', 'X_dendrite_valid.npy', 'X_dendrite_test.npy', 'NAMES_dendrite_train.npy', 'NAMES_dendrite_valid.npy', 'NAMES_dendrite_test.npy']

for tensor, name in zip(tensor2save, names):
    np.save(in_dir+os.sep+name, tensor)