#!/usr/bin/env python

"""
riverAugmenting.py

main program to augment river png images subsets for training.
It uses numpy arrays with images, ALREADY CREATED by the script
createRiverDatasets.py. 

This is the first script to launch for dataset creation.

SEQUENCE OF SCRIPT EXECUTION:

1. (mandatory)	loadRiverImages.py
2. (mandatory)	createRiverDatasets.py
3. (mandatory)	riverAugmenting.py
4. (optional)	saveRiverImages.py

# Copyright (C) 2020-2022 DAME Project
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation;
# either version 3.0 of the License, or (at your option)
# any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to
# the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301 USA
#

WHO                 WHEN         WHAT
------------------- ----------	 -------------------------------------------------
M. Brescia			19/02/2020	 Created
M. Brescia          21/02/2020   bug fixed with names of augmented imgs
-----------------------------------------------------------------------------------
"""

import numpy as np
import os
import random
from functools import partial
import matplotlib.pyplot as plt

in_dir = 'exp'


numPixels = 540 # squared images
mydpi = 1200
perc_augmenting = 1.0 # decimal percentage of augmented images
# total number of samples shown at the end = numSampleRows x numSampleColumns
numSampleRows = 4
numSampleColumns = 8

X_mix_train = np.load(in_dir+os.sep+'X_mix_train.npy')
X_mix_valid = np.load(in_dir+os.sep+'X_mix_valid.npy')
X_mix_test = np.load(in_dir+os.sep+'X_mix_test.npy')
X_dendrite_train = np.load(in_dir+os.sep+'X_dendrite_train.npy')
X_dendrite_valid = np.load(in_dir+os.sep+'X_dendrite_valid.npy')
X_dendrite_test = np.load(in_dir+os.sep+'X_dendrite_test.npy')
Names_mix_train = np.load(in_dir+os.sep+'NAMES_mix_train.npy')
Names_mix_valid = np.load(in_dir+os.sep+'NAMES_mix_valid.npy')
Names_mix_test = np.load(in_dir+os.sep+'NAMES_mix_test.npy')
Names_dendrite_train = np.load(in_dir+os.sep+'NAMES_dendrite_train.npy')
Names_dendrite_valid = np.load(in_dir+os.sep+'NAMES_dendrite_valid.npy')
Names_dendrite_test = np.load(in_dir+os.sep+'NAMES_dendrite_test.npy')

#assume that we want to augment only the 50% of the training set.
#Note that there are 3 rotations and 2 flips, 
#so a process that performs all the transformations increases the training set. 


def showAugmented(X, num_rows, num_cols):
	index = random.sample(range(X.shape[0]), num_rows*num_cols)
	subX = X[index]
	print("index of extracted river images: ")
	print(index)

	#fig, axes = plt.subplots(nrows=num_rows, ncols=num_cols, figsize=(1000/320, 400/320), dpi=320)
	plt.style.use('grayscale')
	fig, axes = plt.subplots(nrows=num_rows, ncols=num_cols, figsize=(numPixels/mydpi, numPixels/mydpi), dpi=mydpi)
	i = 0
	for il in range(num_rows):
		for j in range(num_cols):
			img = X[i].reshape((numPixels, numPixels))
			ax = axes[il,j]
			ax.imshow(img, interpolation='nearest')
			ax.axis('off')
			i = i+1
	plt.show()


def augment(img, N):
	imgs = [img]
	Ns   = [N]
	# k rotations of (k x 90)° counter-clockwise
	imgs.append(np.rot90(img, k=1)) # 90°
	newName = N + '_90'
	Ns.append(newName)
	#print("a. " + newName)

	imgs.append(np.rot90(img, k=2)) # 180°
	newName = N + '_180'
	Ns.append(newName)
	#print("b. " + newName)

	imgs.append(np.rot90(img, k=3)) # 270°
	newName = N + '_270'
	Ns.append(newName)
	#print("c. " + newName)

	# up-down flip
	imgs.append(np.flipud(img))
	newName = N + '_ud'
	Ns.append(newName)
	#print("d. " + newName)

	# left-right flip
	imgs.append(np.fliplr(img))        
	newName = N + '_lr'
	Ns.append(newName)
	#print("e. " + newName)

	return np.array(imgs), np.array(Ns)
	
	

def preprocess(X, N, fract=0.25):
	#print("1. X: " + str(X.shape))
	#print("1. N: " + str(N.shape))
	index = random.sample(range(X.shape[0]), int(X.shape[0]*fract))
	X2aug = X[index]
	N2aug = N[index]
	#print("2. X2aug: " + str(X2aug.shape))
	#print("2. N2aug: " + str(N2aug.shape))
	mask = np.ones(X.shape[0], dtype=bool)
	mask[index] = False
	X = X[mask] #this is the rest of X, i.e. source will not augmented
	N = N[mask] #same for names
	#print("3. X: " + str(X.shape))
	#print("3. N: " + str(N.shape))
         
	XL = []
	NL = []
	for i in range(X.shape[0]):
		XL.append(X[i])
		NL.append(N[i])
	#print("4. XL: " + str(len(XL)))
	#print("4. NL: " + str(len(NL)))
	for il in range(X2aug.shape[0]):
		X_aug, N_aug = augment(X2aug[il], N2aug[il])
		#print("5. X_aug: " + str(len(X_aug)))
		#print("5. N_aug: " + str(len(N_aug)))
		for j in range(X_aug.shape[0]):
			img = X_aug[j].reshape((numPixels, numPixels, 1))
			XL.append(img)
			NL.append(N_aug[j])
	XL = np.array(XL)
	NL = np.array(NL)
	#print("6. XL: " + str(XL.shape))
	#print("6. NL: " + str(NL.shape))
	#XL = np.expand_dims(XL, axis=3)
	return XL, NL
 

if __name__ == '__main__':
	#clearly only the training set has to be augmented.
	#Validating (or testing) on duplicates doesn't make any sense
	print("\nbefore augmenting - training mixed: %d"%X_mix_train.shape[0])
	print("\nbefore augmenting - training dendrite: %d"%X_dendrite_train.shape[0])

	X_mix_train, Names_mix_train = preprocess(X_mix_train, Names_mix_train, perc_augmenting)
	X_dendrite_train, Names_dendrite_train = preprocess(X_dendrite_train, Names_dendrite_train, perc_augmenting)
	#X_mix_train.shape
	print("\nafter augmenting - training mixed: %d"%X_mix_train.shape[0])
	print("\nafter augmenting - training dendrite: %d"%X_dendrite_train.shape[0])
	print(X_mix_train.shape)
	print(X_dendrite_train.shape)
	#print(X_dendrite_train.shape[0]/(X_mix_train.shape[0]+X_dendrite_train.shape[0]))
	X_train = np.concatenate([X_mix_train, X_dendrite_train], axis=0)
	y_train = np.array([1]*X_mix_train.shape[0]+[0]*X_dendrite_train.shape[0])
	names_train = np.concatenate([Names_mix_train,Names_dendrite_train])

	X_valid = np.concatenate([X_mix_valid, X_dendrite_valid], axis=0)
	y_valid = np.array([1]*X_mix_valid.shape[0]+[0]*X_dendrite_valid.shape[0])
	names_valid = np.concatenate([Names_mix_valid,Names_dendrite_valid])
	
	X_test = np.concatenate([X_mix_test, X_dendrite_test], axis=0)
	y_test = np.array([1]*X_mix_test.shape[0]+[0]*X_dendrite_test.shape[0])
	names_test = np.concatenate([Names_mix_test,Names_dendrite_test])

	#shuffling:
	index = random.sample(range(y_train.shape[0]), y_train.shape[0])
	X_train = X_train[index]
	y_train = y_train[index]
	names_train = names_train[index]

	print("Train X    : " + str(X_train.shape[0]) + " [" + str(X_train.shape) + "]")
	print("Train y    : " + str(y_train.shape[0]) + " [" + str(y_train.shape) + "]")
	print("Train names: " + str(names_train.shape[0]) + " [" + str(names_train.shape) + "]")

	index = random.sample(range(y_valid.shape[0]), y_valid.shape[0])
	X_valid = X_valid[index]
	y_valid = y_valid[index]
	names_valid = names_valid[index]

	print("Train valid X    : " + str(X_valid.shape[0]) + " [" + str(X_valid.shape) + "]")
	print("Train valid y    : " + str(y_valid.shape[0]) + " [" + str(y_valid.shape) + "]")
	print("Train valid names: " + str(names_valid.shape[0]) + " [" + str(names_valid.shape) + "]")

	index = random.sample(range(y_test.shape[0]), y_test.shape[0])
	X_test = X_test[index]
	y_test = y_test[index]
	names_test = names_test[index]

	print("Test X    : " + str(X_test.shape[0]) + " [" + str(X_test.shape) + "]")
	print("Test y    : " + str(y_test.shape[0]) + " [" + str(y_test.shape) + "]")
	print("Test names: " + str(names_test.shape[0]) + " [" + str(names_test.shape) + "]")

	np.save(in_dir+os.sep+'X_train.npy', X_train)
	np.save(in_dir+os.sep+'y_train.npy', y_train)
	np.save(in_dir+os.sep+'names_train.npy', names_train)
	np.save(in_dir+os.sep+'X_valid.npy', X_valid)
	np.save(in_dir+os.sep+'y_valid.npy', y_valid)
	np.save(in_dir+os.sep+'names_valid.npy', names_valid)
	np.save(in_dir+os.sep+'X_test.npy', X_test)
	np.save(in_dir+os.sep+'y_test.npy', y_test)
	np.save(in_dir+os.sep+'names_test.npy', names_test)
	showAugmented(X_train, numSampleRows, numSampleColumns)
	
	